/* $Id: sg_config.c,v 1.3 1996/03/26 22:41:51 james Exp james $ */
/*
 * $Log: sg_config.c,v $
 * Revision 1.3  1996/03/26 22:41:51  james
 * We now use library routines for decoding and encoding the config
 * byte.  We remove the global variables, replacing them with a struct as
 * used in the interface to the library functions, which struct becomes
 * local to main().
 *
 * Also, support setting the DSP/Mixer address.
 *
 * Revision 1.2  1996/03/25 02:17:08  james
 * Fairly tested; the DSP address field is not yet supported.
 *
 * Revision 1.1  1996/03/25 02:14:33  james
 * Initial revision
 *
 */

/*
 * TODO: Support the "DSP address" field.
 */

#include <stdio.h>		/* perror() */
#include <stdlib.h>		/* atoi() */
#include <string.h>		/* strdup() */
#include <assert.h>		/* assert() */
#include <limits.h>		/* CHAR_BIT */
#include <getopt.h>		/* getopt_long() */
#include "jy_sg.h"

#define PORT_IO_DEVICE "/dev/port" /* We may need setgid kmem. */

#define ASSERT(x) assert(x)

char *progname;

/* 
 * Data governing the command-line options.
 */
static const char short_opts[] = "d:i:";
enum { OptCd, OptCdAddr, OptDspAddr, OptGameport, OptDma, OptIrq, OptTest };

static struct option long_opts[] =
{
  /* name    has_arg    int *flag,    val  */
  { "cd",	1,	0,		0 },
  { "cdaddr",	1,	0,		0 },
  { "dspaddr",	1,	0,		0 },
  { "gameport",	1,	0,		0 },
  { "dma",	1,	0,	        0 },
  { "irq",	1,	0,	        0 },
  { "test",	0,	0,	0 },

  { 0,		0,	0,		0 } /* end marker */
};

static void
usage(void)
{
  extern char *progname;
  
  fprintf(stderr,
	  "usage: %s [options...]\n"
	  "Options are:-\n"
	  "\t--cd=AT	    Configure for AT-type interface (Panasonic etc.)\n"
	  "\t--cd=SCSI     *Configure for SCSI CD-ROM interface\n"
	  "\t--cdaddr=X     Set CD-ROM interface address (*0x300 or 0x320)\n"
	  "\t--dspaddr=X    Set DSP/Mixer address (*0x220 or 0x240)\n"
	  "\t--dma=X	    Set DMA channel (0, *1 or 3)\n"
	  "\t--irq=X	    Set IRQ number (2, 5, *7 or 10)\n"
	  "\t--gameport=on *Enable the Game Port\n"
	  "\t--gameport=off Disable the Game Port\n"
	  "\t--test	    Test only (don\'t write to EEPROM)\n"
	  "\n"
	  "Above, the * indicates the setting that is the card\'s default.\n"
	  "Options which are not specified on the command line are not\n"
	  "changed on the card, so specifying just one option changes just\n"
	  "that option only.\n",
	  progname);
}

/* Functions for setting options.  If the argument is wrong,
 * the function calls bad_usage(), which exits the program.
 * Otherwise, the return value is nonzero if that configuration 
 * item was changed.
 */
typedef int optfunc(struct sgnx_eeprom_config *p, const char* arg);
optfunc set_cdtype, set_cdaddr, set_dspaddr, set_gameport, set_dma, set_irq;

/* Other functions */
static void bad_usage(void);


int
main(int argc, char *argv[])
{
  struct sgnx_eeprom_config cfg;
  int need_write;
  int test_only;
  BYTE config_byte;
  
  progname = argv[0];
  
  if (argc == 1)
    {
      usage();
      exit(0);
    }
  
  test_only = 0;
  
  /* Read the data from the EEPROM. */
  config_byte = 0;

  read_eeprom_block(&config_byte, 1);
  need_write = 0;

  if (!decode_sgnx_config_byte(config_byte, &cfg))
    {
      fprintf(stderr,
	      "Dont\'t understand configuration byte value 0x%02X\n",
	      config_byte);
      return 1;
    }
  
  /* Process the options */
  while (1)
    {
      int optind;
      int c;
      extern char *optarg;
      
      c = getopt_long(argc, argv, short_opts, long_opts, &optind);
      if (c == -1)
	break;
      switch (c)		/* cope with short options */
	{
	case 0:			/* long option */
	  switch (optind)
	    {
	    case OptCd:
	      need_write |= set_cdtype(&cfg, optarg);
	      break;
	    case OptCdAddr:
	      need_write |= set_cdaddr(&cfg, optarg);
	      break;
	    case OptDspAddr:
	      need_write |= set_dspaddr(&cfg, optarg);
	      break;
	    case OptGameport:
	      need_write |= set_gameport(&cfg, optarg);
	      break;
	    case OptDma:
	      need_write |= set_dma(&cfg, optarg);
	      break;
	    case OptIrq:
	      need_write |= set_irq(&cfg, optarg);
	      break;
	    case OptTest:
	      test_only = 1;
	      break;
	    }
	  break;

	case 'd':
	  need_write |= set_dma(&cfg, optarg);
	  break;
	  
	case 'i':
	  need_write |= set_irq(&cfg, optarg);
	  break;
	}
    }      
  /* Done processing options. */
  fprintf(stderr, "Done with options.\n");
  if (test_only)
    fprintf(stderr, "Test-only mode.\n");

  if (need_write)
    {
      if (!encode_sgnx_config_byte(&cfg, &config_byte))
	{
	  /* This should never happen */
	  fprintf(stderr,
		  "Invalid configuration "
		  "[CD at 0x%03X, DSP at 0x%03X, DMA=%d, IRQ=%d]\n"
		  "(this should never happen).\n",
		  cfg.cdrom_addr, cfg.dsp_and_mixer_addr, cfg.dma, cfg.irq);
	  return 1;
	}
      
      if (test_only)
	{
	  fprintf(stdout, "Config byte value would be written as %02x:\n",
		  (int)config_byte);
	  
	  fprintf(stdout, "%s\n", decode_sgnx_config(&cfg));
	  fflush(stdout);
	}
      else
	{
	  if (need_write)
	    {
	      write_eeprom_block(&config_byte, 1);
	    }
	  else
	    {
	    fprintf(stderr, "EEPROM configuration unchanged (0x%02X)",
		    config_byte);
	    }
	}
    }
  exit(0);
}

int
set_cdtype(struct sgnx_eeprom_config *p, const char *arg)
{
  if (arg)
    {
      if (0 == strcasecmp(arg, "scsi"))
	{
	  p->cd_is_atbus = 0;
	  return (0==p->cd_is_atbus);
	}
      else if (0 == strcasecmp(arg, "at"))
	{
	  p->cd_is_atbus = 1;
	  return (!p->cd_is_atbus);
	}
    }
  /* else incorrect usage. */
  bad_usage();
  
  /* NOTREACHED */
  return 0;			/* satisfy the compiler */
}

int
set_cdaddr(struct sgnx_eeprom_config *p, const char *arg)
{
  /* arg must be 0x300 or 0x320. */
  int val = strtoul(arg, NULL, 16);

  if ( (val == 0x320) || (val == 0x300) )
    {
      int old_addr = p->cdrom_addr;
      p->cdrom_addr = val;
      return (p->cdrom_addr != old_addr);
    }
  /* Otherwise... */
  bad_usage();
  
  /* NOTREACHED */
  return 0;			/* satisfy the compiler */
}


int
set_dspaddr(struct sgnx_eeprom_config *p, const char *arg)
{
  /* arg must be 0x220 or 0x240. */
  int val = strtoul(arg, NULL, 16);

  if ( (val == 0x220) || (val == 0x240) )
    {
      int old_addr = p->dsp_and_mixer_addr;
      p->dsp_and_mixer_addr = val;
      return (p->dsp_and_mixer_addr != old_addr);
    }
  /* Otherwise... */
  bad_usage();
  
  /* NOTREACHED */
  return 0;			/* satisfy the compiler */
}


int
set_gameport(struct sgnx_eeprom_config *p, const char *arg)
{
  /* arg must be "enabled" or "disabled" */
  if (arg)
    {
      int need_write;
      if ( (0 == strcasecmp(arg, "enabled")) ||
	   (0 == strcasecmp(arg, "on"))       )
	{
	  need_write = (! p->game_port_enabled);
	  p->game_port_enabled = 1;
	  return need_write;
	}
      else if ( (0 == strcasecmp(arg, "disabled")) ||
	        (0 == strcasecmp(arg, "off"))       )
	{
	  need_write = (0 == p->game_port_enabled);
	  p->game_port_enabled = 0;
	  return need_write;
	}
    }
  bad_usage();
  
  /* NOTREACHED */
  return 0;			/* satisfy the compiler */
}

int
set_dma(struct sgnx_eeprom_config *p, const char *arg)
{
  if (arg)
    {
      const int old_dma = p->dma;
      const int val = atoi(arg);
      switch (val)
	{
	case 0:
	case 1:
	case 3:
	  p->dma = val;
	  return (p->dma != old_dma);
	default:
	  break;
	}
    }
  bad_usage();
  
  /* NOTREACHED */
  return 0;			/* satisfy the compiler */
}

int
set_irq(struct sgnx_eeprom_config *p, const char *arg)
{
  if (arg)
    {
      const int old_irq = p->irq;
      const int val = atoi(arg);
      switch (val)
	{
	case  2:
	case  5:
	case  7:
	case 10:
	  p->irq = val;
	  return (p->irq != old_irq);
	default:
	  break;
	}
    }
  bad_usage();
  
  /* NOTREACHED */
  return 0;			/* satisfy the compiler */
}

static void
bad_usage(void)
{
  usage();
  exit(1);
}

